// 013 u_c_file_list
// 03 jan 2002

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- documentation at http://www.jcolibri.com

// file: the elements
// path: the payload

(*$r+*)

unit u_c_file_list;
  interface
    uses Classes, u_c_basic_object;

    type c_file_list= Class; // forward

         c_file= // one "file"
                Class(c_basic_object)
                  // -- not the same name as the tStringList
                  m_c_parent_file_list: c_file_list;
                  m_path: String;

                  public
                    Constructor create_file(p_name: String; p_c_file_list: c_file_list;
                        p_path: String); Virtual;
                    function f_display_file: String;
                    Destructor Destroy; Override;
                end;

         c_file_list= // "file" list
                 Class(c_basic_object)
                   public
                     m_c_file_list: tStringList;

                     Constructor create_file_list(p_name: String); Virtual;

                     function f_file_count: Integer;
                     function f_c_file(p_file: Integer): c_file;
                     function f_index_of(p_file: String): Integer;
                     function f_c_find_by_file(p_file: String): c_file;
                     procedure add_file(p_file: String; p_c_file: c_file);
                     procedure add_element(p_file, p_path: String);
                     procedure add_unique_file(p_file, p_path: String);
                     procedure display_file_list;

                     Destructor Destroy; Override;
                   end;

  implementation
    uses SysUtils, u_c_display;

    // -- c_file

    Constructor c_file.create_file(p_name: String; p_c_file_list: c_file_list;
        p_path: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_parent_file_list:= p_c_file_list;
        m_path:= p_path;
      end; // create_file

    function c_file.f_display_file: String;
      begin
        Result:= Format('%-10s ', [m_name])+ m_path;
      end; // f_display_file

    Destructor c_file.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_file_list

    Constructor c_file_list.create_file_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_file_list:= tStringList.Create;
      end; // create_file_list

    function c_file_list.f_file_count: Integer;
      begin
        Result:= m_c_file_list.Count;
      end; // f_file_count

    function c_file_list.f_c_file(p_file: Integer): c_file;
      begin
        Result:= c_file(m_c_file_list.Objects[p_file]);
      end; //  f_c_file

    function c_file_list.f_index_of(p_file: String): Integer;
      begin
        Result:= m_c_file_list.IndexOf(p_file);
      end; // f_index_of

    function c_file_list.f_c_find_by_file(p_file: String): c_file;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_file);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_file(m_c_file_list.Objects[l_index_of]);
      end; // f_c_find_by_file

    procedure c_file_list.add_file(p_file: String; p_c_file: c_file);
      begin
        m_c_file_list.AddObject(p_file, p_c_file);
      end; // add_file

    procedure c_file_list.add_element(p_file, p_path: String);
      begin
        m_c_file_list.AddObject(p_file, c_file.create_file(p_file, Self, p_path));
      end; // add_element

    procedure c_file_list.add_unique_file(p_file, p_path: String);
      begin
        if f_index_of(p_file)>= 0
          then
          else m_c_file_list.AddObject(p_file, c_file.create_file(p_file, Self, p_path));
      end; // add_unique_file

    procedure c_file_list.display_file_list;
      var l_file: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_file_count));

        for l_file:= 0 to f_file_count- 1 do
          display('  '+ f_c_file(l_file).f_display_file);
      end; // display_file_list

    Destructor c_file_list.Destroy;
      var l_file: Integer;
      begin
        for l_file:= 0 to f_file_count- 1 do
          f_c_file(l_file).Free;
        m_c_file_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_file
    end. // u_c_file
