// 013 u_c_path_list
// 03 jan 2002

// -- (C) John COLIBRI 2006
// -- documentation: http://www.jcolibri.com

// -- documentation at http://www.jcolibri.com

// path: the elements
// file: the payload

(*$r+*)

unit u_c_path_list;
  interface
    uses Classes, u_c_basic_object;

    type c_path_list= Class; // forward

         c_path= // one "path"
                Class(c_basic_object)
                  // -- not the same name as the tStringList
                  m_c_parent_path_list: c_path_list;
                  m_file: String;

                  public
                    Constructor create_path(p_name: String; p_c_path_list: c_path_list;
                        p_file: String); Virtual;
                    function f_display_path: String;
                    Destructor Destroy; Override;
                end;

         c_path_list= // "path" list
                 Class(c_basic_object)
                   public
                     m_c_path_list: tStringList;
                     m_path_length_max: Integer;

                     Constructor create_path_list(p_name: String); Virtual;

                     function f_path_count: Integer;
                     function f_c_path(p_path: Integer): c_path;
                     function f_index_of(p_path: String): Integer;
                     function f_c_find_by_path(p_path: String): c_path;
                     procedure add_path(p_path: String; p_c_path: c_path);
                     procedure add_element(p_path, p_file: String);
                     procedure add_unique_path(p_path, p_file: String);
                     procedure display_path_list;
                     procedure display_the_path_and_files;

                     Destructor Destroy; Override;
                   end;

  implementation
    uses SysUtils, Math, u_c_display;

    // -- c_path

    Constructor c_path.create_path(p_name: String; p_c_path_list: c_path_list;
        p_file: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_parent_path_list:= p_c_path_list;
        m_file:= p_file;

        with m_c_parent_path_list do
          m_path_length_max:= Max(m_path_length_max, Length(p_name));
      end; // create_path

    function c_path.f_display_path: String;
      begin
        Result:= Format('%-10s ', [m_name])+ m_file;
      end; // f_display_path

    Destructor c_path.Destroy;
      begin
        InHerited;
      end; // Destroy

    // -- c_path_list

    Constructor c_path_list.create_path_list(p_name: String);
      begin
        Inherited create_basic_object(p_name);

        m_c_path_list:= tStringList.Create;
      end; // create_path_list

    function c_path_list.f_path_count: Integer;
      begin
        Result:= m_c_path_list.Count;
      end; // f_path_count

    function c_path_list.f_c_path(p_path: Integer): c_path;
      begin
        Result:= c_path(m_c_path_list.Objects[p_path]);
      end; //  f_c_path

    function c_path_list.f_index_of(p_path: String): Integer;
      begin
        Result:= m_c_path_list.IndexOf(p_path);
      end; // f_index_of

    function c_path_list.f_c_find_by_path(p_path: String): c_path;
      var l_index_of: Integer;
      begin
        l_index_of:= f_index_of(p_path);
        if l_index_of< 0
          then Result:= Nil
          else Result:= c_path(m_c_path_list.Objects[l_index_of]);
      end; // f_c_find_by_path

    procedure c_path_list.add_path(p_path: String; p_c_path: c_path);
      begin
        m_c_path_list.AddObject(p_path, p_c_path);
      end; // add_path

    procedure c_path_list.add_element(p_path, p_file: String);
      begin
        m_c_path_list.AddObject(p_path, c_path.create_path(p_path, Self, p_file));
      end; // add_element

    procedure c_path_list.add_unique_path(p_path, p_file: String);
      begin
        if f_index_of(p_path)>= 0
          then
          else m_c_path_list.AddObject(p_path, c_path.create_path(p_path, Self, p_file));
      end; // add_unique_path

    procedure c_path_list.display_path_list;
      var l_path: Integer;
      begin
        display(m_name+ ' '+ IntToStr(f_path_count));

        for l_path:= 0 to f_path_count- 1 do
          display(f_c_path(l_path).f_display_path);
      end; // display_path_list

    procedure c_path_list.display_the_path_and_files;
      var l_path_and_file: Integer;
          l_path, l_previous_path: String;
      begin
        display(m_name+ ' '+ IntToStr(f_path_count));

        l_previous_path:= '';
        for l_path_and_file:= 0 to f_path_count- 1 do
          with f_c_path(l_path_and_file) do
          begin
            l_path:= m_name;
            if l_path= l_previous_path
              then l_path:= '';
            display(Format('%-'+ IntToStr(m_path_length_max)+ 's ', [l_path])
              + m_file);
            l_previous_path:= m_name;
          end; // for, with
      end; // display_the_path_and_files

    Destructor c_path_list.Destroy;
      var l_path: Integer;
      begin
        for l_path:= 0 to f_path_count- 1 do
          f_c_path(l_path).Free;
        m_c_path_list.Free;

        Inherited;
      end; // Destroy

    begin // u_c_path
    end. // u_c_path
